package com.zilber.boot.module.booking.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.zilber.boot.Constants.HttpStatus;
import com.zilber.boot.module.booking.dao.BookingMapper;
import com.zilber.boot.module.booking.enums.ApprovalStatus;
import com.zilber.boot.module.booking.enums.BookingType;
import com.zilber.boot.module.booking.form.ApprovalForm;
import com.zilber.boot.module.booking.form.add.BookingAF;
import com.zilber.boot.module.booking.form.modify.BookingMF;
import com.zilber.boot.module.booking.form.pr.BookingPR;
import com.zilber.boot.module.booking.group.Me;
import com.zilber.boot.module.booking.group.Reject;
import com.zilber.boot.module.booking.pojo.Booking;
import com.zilber.boot.module.booking.service.BookingService;
import com.zilber.boot.module.booking.vo.BookingVO;
import com.zilber.boot.utils.InternalResult;
import com.zilber.boot.utils.SecurityUtils;
import com.zilber.boot.utils.ValidatorUtils;
import com.zilber.boot.utils.page.PageUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;

import java.io.Serializable;
import java.util.Calendar;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;

/**
 * 预约记录(Booking)表服务实现类
 *
 * @author liuao
 * @since 2025-04-25 15:50:01
 */
@Service
public class BookingServiceImpl
        extends ServiceImpl<BookingMapper, Booking>
        implements BookingService
{

    /**
     * 申请预约
     * @param form 预约表单
     * @return 处理结果
     */
    @Override
    public InternalResult addBooking(BookingAF form)
    {
        ValidatorUtils.validateEntity(form);
        //如果是省招，则校验中标信息
        if ( form.getBookingType().equals(BookingType.PRT))
        {
            ValidatorUtils.validateEntity(form, Me.class);
        }
        Long userId = SecurityUtils.getUserId();
        Booking booking = new Booking(form, userId);
        this.save(booking);
        return new InternalResult(HttpStatus.SUCCESS, "预约申请成功");
    }

    /**
     * 修改预约信息
     * @param form 修改表单
     * @return 修改结果
     */
    @Override
    public InternalResult updBooking(BookingMF form)
    {
        ValidatorUtils.validateEntity(form);
        Booking byId = this.getById(form.getId());
        if ( byId != null)
        {
            BeanUtils.copyProperties(form, byId);
            this.updateById(byId);
        }
        return new InternalResult(HttpStatus.SUCCESS, "预约信息修改成功");
    }

    /**
     * 审批预约
     * @param form 预约表单
     * @return 处理结果
     */
    @Override
    public InternalResult approve(ApprovalForm form)
    {
        ValidatorUtils.validateEntity(form);
        if ( form.getApprovalStatus().equals(ApprovalStatus.NONE))
        {
            return new InternalResult(HttpStatus.SUCCESS, "未作出审批");
        }
        else if ( form.getApprovalStatus().equals(ApprovalStatus.REJECT))
        {
            ValidatorUtils.validateEntity(form, Reject.class);
        }
        Booking byId = this.getById(form.getId());
        if ( byId != null)
        {
            byId.setApprovalStatus(form.getApprovalStatus().getCode());
            byId.setRejectReason(form.getRejectReason());
            byId.setApproveTime(new Date());
            this.updateById(byId);
        }
        return new InternalResult(HttpStatus.SUCCESS, "审批成功");
    }

    /**
     * 获取单条预约信息
     * @param id id
     * @return 单条预约记录
     */
    @Override
    public BookingVO getBooking(Serializable id)
    {
        Booking byId = this.getById(id);
        if ( byId != null)
        {
            return new BookingVO(byId);
        }
        return null;
    }

    /**
     * 分页查询
     * @param pr 查询条件
     * @return 分页列表
     */
    @Override
    public PageUtils getPage(BookingPR pr)
    {
        LambdaQueryWrapper<Booking> wrapper = this.wrapper(pr);
        IPage<Booking> page
                = page(new Page(pr.getPageNo(), pr.getPageSize()), wrapper);
        if ( CollectionUtils.isNotEmpty(page.getRecords()))
        {
            return new PageUtils(
                    page.getRecords()
                            .stream()
                            .map(BookingVO::new)
                            .collect(Collectors.toList()),
                    (int) page.getTotal(),
                    (int) page.getSize(),
                    (int) page.getCurrent()
            );
        }
        return new PageUtils(page);
    }

    /**
     * 删除单条预约记录
     * @param form 表单
     * @return 处理结果
     */
    @Override
    public InternalResult deleteBooking(BookingMF form)
    {
        ValidatorUtils.validateEntity(form);
        this.removeById(form.getId());
        return new InternalResult(HttpStatus.SUCCESS, "预约记录删除成功");
    }

    /**
     * 删除多条预约记录
     * @param idList id列表
     * @return 处理结果
     */
    @Override
    public InternalResult deleteBookings(List<Serializable> idList)
    {
        if (CollectionUtils.isNotEmpty(idList))
        {
            LambdaQueryWrapper<Booking> wrapper = new LambdaQueryWrapper<>();
            wrapper.in(Booking::getId, idList);
            this.remove(wrapper);
        }
        return new InternalResult(HttpStatus.SUCCESS, "预约记录删除成功");
    }

    /**
     * 根据查询分页构造查询包装器并返回
     * @param pr 查询条件
     * @return 查询包装器
     */
    private LambdaQueryWrapper<Booking> wrapper(BookingPR pr)
    {
        LambdaQueryWrapper<Booking> wrapper = new LambdaQueryWrapper<>();
        if ( pr.getFactoryId() != null)
        {
            wrapper.eq(Booking::getFactoryId, pr.getFactoryId());
        }
        if (StringUtils.isNotBlank(pr.getBookingType()))
        {
            wrapper.eq(Booking::getBookingType, pr.getBookingType());
        }
        if ( StringUtils.isNotBlank(pr.getInspectionMethod()))
        {
            wrapper.eq(Booking::getInspectionMethod, pr.getInspectionMethod());
        }
        if ( pr.getEquipNum() != null)
        {
            wrapper.eq(Booking::getEquipNum, pr.getEquipNum());
        }
        if ( StringUtils.isNotBlank(pr.getEquipType()))
        {
            wrapper.eq(Booking::getEquipType, pr.getEquipType());
        }
        Calendar c = Calendar.getInstance();
        if ( pr.getInspectionDateStart() != null)
        {
            c.setTime(pr.getInspectionDateStart());
            c.set(Calendar.HOUR_OF_DAY, 0);
            c.set(Calendar.MINUTE, 0);
            c.set(Calendar.SECOND, 0);
            wrapper.ge(Booking::getInspectionDate, c.getTime());
        }
        if ( pr.getInspectionDateEnd() != null)
        {
            c.setTime(pr.getInspectionDateEnd());
            c.set(Calendar.HOUR_OF_DAY, 0);
            c.set(Calendar.MINUTE, 0);
            c.set(Calendar.SECOND, 0);
            wrapper.le(Booking::getInspectionDate, c.getTime());
        }
        if ( StringUtils.isNotBlank(pr.getLocalOffice()))
        {
            wrapper.like(Booking::getLocalOffice, pr.getLocalOffice());
        }
        if ( StringUtils.isNotBlank(pr.getLoaType()))
        {
            wrapper.eq(Booking::getLoaType, pr.getLoaType());
        }
        if (StringUtils.isNotBlank(pr.getProjectName()))
        {
            wrapper.like(Booking::getProjectName, pr.getProjectName());
        }
        if ( pr.getBidBatch() != null)
        {
            wrapper.eq(Booking::getBidBatch, pr.getBidBatch());
        }
        if ( pr.getCreatorId() != null)
        {
            wrapper.eq(Booking::getCreatorId, pr.getCreatorId());
        }
        if ( StringUtils.isNotBlank(pr.getApprovalStatus()))
        {
            wrapper.eq(Booking::getApprovalStatus, pr.getApprovalStatus());
        }
        if ( pr.getApproveTimeStart() != null)
        {
            c.setTime(pr.getApproveTimeStart());
            c.set(Calendar.HOUR_OF_DAY, 0);
            c.set(Calendar.MINUTE, 0);
            c.set(Calendar.SECOND, 0);
            wrapper.ge(Booking::getApproveTime, c.getTime());
        }
        if ( pr.getApproveTimeEnd() != null)
        {
            c.setTime(pr.getInspectionDateEnd());
            c.set(Calendar.HOUR_OF_DAY, 23);
            c.set(Calendar.MINUTE, 59);
            c.set(Calendar.SECOND, 59);
            wrapper.le(Booking::getApproveTime, c.getTime());
        }
        return wrapper;
    }

}
